#requires -Version 5.1
Add-Type -AssemblyName System.Windows.Forms
Add-Type -AssemblyName System.Drawing

# Percorso file config
$ScriptDir = Split-Path -Parent $MyInvocation.MyCommand.Definition
$jsonPath = Resolve-Path (Join-Path $ScriptDir "..\..\..\resources\pdca0-config.json")
if (!(Test-Path $jsonPath)) {
    [System.Windows.Forms.MessageBox]::Show("File di configurazione non trovato: $jsonPath", "Errore", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Error)
    exit 1
}

. "$ScriptDir\..\..\general\get-terminal-positions.ps1"

$pdca0Path = Join-Path $ScriptDir "..\..\..\..\pdca0"


# Lettura moduli dal file JSON
$json = Get-Content $jsonPath -Raw | ConvertFrom-Json
$modules = @()
if ($json.backend_modules) { $modules += $json.backend_modules }
if ($json.core_module) { $modules += $json.core_module }
$modules = $modules | Sort-Object -Unique

if ($modules.Count -eq 0) {
    [System.Windows.Forms.MessageBox]::Show("Nessun modulo trovato nel file di configurazione.", "Errore", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Error)
    exit 1
}

# Crea form in stile setup-conf.ps1
$form = New-Object System.Windows.Forms.Form
$form.Text = "Darwin PDCA0 Backend Util"
$form.Size = New-Object System.Drawing.Size(600, 500)
$form.StartPosition = "CenterScreen"

# TableLayoutPanel principale
$mainPanel = New-Object System.Windows.Forms.TableLayoutPanel
$mainPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
$mainPanel.RowCount = 2
$mainPanel.ColumnCount = 1
$mainPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Percent, 100)))
$mainPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Absolute, 35)))
$form.Controls.Add($mainPanel)

# Pannello bianco con bordo per i checkbox
$dictPanel = New-Object System.Windows.Forms.Panel
$dictPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
$dictPanel.BorderStyle = 'Fixed3D'
$dictPanel.BackColor = 'White'
$dictPanel.AutoScroll = $true
$dictPanel.Anchor = [System.Windows.Forms.AnchorStyles]::Top -bor [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Left -bor [System.Windows.Forms.AnchorStyles]::Right
$mainPanel.Controls.Add($dictPanel, 0, 0)

# Suddividi moduli in core e modules
$coreModules = @()
$backendModules = @()
if ($json.core_module) { $coreModules += $json.core_module }
if ($json.backend_modules) { $backendModules += $json.backend_modules }
$coreModules = $coreModules | Sort-Object -Unique
$backendModules = $backendModules | Sort-Object -Unique

$checkboxes = @()
$y = 10
# Label core_module
$coreLabel = New-Object System.Windows.Forms.Label
$coreLabel.Text = "core_module"
$coreLabel.Font = New-Object System.Drawing.Font('Segoe UI', 9, [System.Drawing.FontStyle]::Regular)
$coreLabel.Location = New-Object System.Drawing.Point(10, $y)
$coreLabel.Size = New-Object System.Drawing.Size(200, 20)
$dictPanel.Controls.Add($coreLabel)
$y += 22
# Checkbox core
foreach ($mod in $coreModules) {
    $cb = New-Object System.Windows.Forms.CheckBox
    $cb.Text = $mod
    $cb.Font = New-Object System.Drawing.Font('Segoe UI', 9, [System.Drawing.FontStyle]::Regular)
    $cb.Location = New-Object System.Drawing.Point(30, $y)
    $cb.Size = New-Object System.Drawing.Size(510, 22)
    $cb.Checked = $true
    $dictPanel.Controls.Add($cb)
    $checkboxes += $cb
    $y += 24
}
$y += 10
# Label backend_modules
$modulesLabel = New-Object System.Windows.Forms.Label
$modulesLabel.Text = "backend_modules"
$modulesLabel.Font = New-Object System.Drawing.Font('Segoe UI', 9, [System.Drawing.FontStyle]::Regular)
$modulesLabel.Location = New-Object System.Drawing.Point(10, $y)
$modulesLabel.Size = New-Object System.Drawing.Size(200, 20)
$dictPanel.Controls.Add($modulesLabel)
$y += 22
# Checkbox modules
foreach ($mod in $backendModules) {
    $cb = New-Object System.Windows.Forms.CheckBox
    $cb.Text = $mod
    $cb.Font = New-Object System.Drawing.Font('Segoe UI', 9, [System.Drawing.FontStyle]::Regular)
    $cb.Location = New-Object System.Drawing.Point(30, $y)
    $cb.Size = New-Object System.Drawing.Size(510, 22)
    $cb.Checked = $true
    $dictPanel.Controls.Add($cb)
    $checkboxes += $cb
    $y += 24
}

# Pannello per il bottone in basso
$buttonPanel = New-Object System.Windows.Forms.Panel
$buttonPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
$buttonPanel.Anchor = [System.Windows.Forms.AnchorStyles]::Top -bor [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Left -bor [System.Windows.Forms.AnchorStyles]::Right
$buttonPanel.Height = 45
$mainPanel.Controls.Add($buttonPanel, 0, 1)

# Bottone installa
$btnInstall = New-Object System.Windows.Forms.Button
$btnInstall.Text = "Installa selezionati"
$btnInstall.Size = New-Object System.Drawing.Size(120, 25)
$btnInstall.Font = New-Object System.Drawing.Font('Segoe UI', 9, [System.Drawing.FontStyle]::Regular)
$btnInstall.Anchor = [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Right
$btnInstall.Location = New-Object System.Drawing.Point(320, 0)
$buttonPanel.Controls.Add($btnInstall)

# Bottone avvia
$btnStart = New-Object System.Windows.Forms.Button
$btnStart.Text = "Avvia selezionati"
$btnStart.Size = New-Object System.Drawing.Size(120, 25)
$btnStart.Font = New-Object System.Drawing.Font('Segoe UI', 9, [System.Drawing.FontStyle]::Regular)
$btnStart.Anchor = [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Right
$btnStart.Location = New-Object System.Drawing.Point(450, 0)
$buttonPanel.Controls.Add($btnStart)

# Azione installazione
$btnInstall.Add_Click({
    $selected = $checkboxes | Where-Object { $_.Checked } | ForEach-Object { $_.Text }
    if ($selected.Count -eq 0) {
        [System.Windows.Forms.MessageBox]::Show("Seleziona almeno un modulo.", "Attenzione", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Warning)
        return
    }
    $form.Enabled = $false
    $positions = Get-TerminalPositions $modules.Count
    foreach ($mod in $selected) {
        $index = [Array]::IndexOf($modules, $mod)
        if ($index -lt 0) { continue }
        $modPathRaw = Join-Path $pdca0Path $mod
        try {
            $modPath = Resolve-Path $modPathRaw -ErrorAction Stop | Select-Object -ExpandProperty Path
        } catch {
            [System.Windows.Forms.MessageBox]::Show("Modulo non trovato: $modPathRaw", "Errore", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Error)
            continue
        }
        $startCmdPosScript = Join-Path $ScriptDir "..\..\general\start-cmd-pos.ps1"
        $cmd = "cd /d $ScriptDir && install.bat `"" + $modPath + "`""
        $pos = $positions[$index]
        powershell -ExecutionPolicy Bypass -File $startCmdPosScript -CommandLine $cmd -X $pos.X -Y $pos.Y -Width $pos.Width -Height $pos.Height
    }
    [System.Windows.Forms.MessageBox]::Show("Installazione avviata in nuove finestre terminale.", "Info", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Information)
    $form.Enabled = $true
})

# Azione avvio
$btnStart.Add_Click({
    $selected = $checkboxes | Where-Object { $_.Checked } | ForEach-Object { $_.Text }
    if ($selected.Count -eq 0) {
        [System.Windows.Forms.MessageBox]::Show("Seleziona almeno un modulo.", "Attenzione", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Warning)
        return
    }
    $form.Enabled = $false
    $positions = Get-TerminalPositions $modules.Count
    foreach ($mod in $selected) {
        $index = [Array]::IndexOf($modules, $mod)
        if ($index -lt 0) { continue }
        $isCore = $coreModules -contains $mod
        if ($isCore) {
            $batName = "start_core.bat"
        } else {
            $batName = "start.bat"
        }
        $modPathRaw = Join-Path $pdca0Path $mod
        try {
            $modPath = Resolve-Path $modPathRaw -ErrorAction Stop | Select-Object -ExpandProperty Path
        } catch {
            [System.Windows.Forms.MessageBox]::Show("Modulo non trovato: $modPathRaw", "Errore", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Error)
            continue
        }
        $startCmdPosScript = Join-Path $ScriptDir "..\..\general\start-cmd-pos.ps1"
        $cmd = "cd /d $ScriptDir && $batName `"" + $modPath + "`""
        $pos = $positions[$index]
        powershell -ExecutionPolicy Bypass -File $startCmdPosScript -CommandLine $cmd -X $pos.X -Y $pos.Y -Width $pos.Width -Height $pos.Height
    }
    [System.Windows.Forms.MessageBox]::Show("Avvio progetti in nuove finestre terminale.", "Info", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Information)
    $form.Enabled = $true
})

# Avvia la UI
[void]$form.ShowDialog()
