# Script PowerShell: setup-conf.ps1
# 1. Recupera i nomi delle cartelle
# 2. Riorganizza secondo le regole
# 3. Legge e aggiorna il JSON con una UI interattiva

Add-Type -AssemblyName System.Windows.Forms
Add-Type -AssemblyName System.Drawing

$ScriptDir = Split-Path -Parent $MyInvocation.MyCommand.Definition

$basePath = Resolve-Path (Join-Path $ScriptDir '../../../../pdca0')

# 1. Recupera i nomi delle cartelle
$dirs = Get-ChildItem -Path $basePath -Directory | Select-Object -ExpandProperty Name

# 2. Riorganizza gli input
$projects = $dirs | Where-Object { $_ -like 'xdce-module*' -or $_ -like '*-ui' -or $_ -like '*deploy*' -or $_ -like '*arch-impl*' }
$packages = $dirs | Where-Object { $_ -like 'xdce-module*' -or $_ -like '*-ui' -or $_ -like '*arch-impl*' }

# Funzione per creare la UI dinamica
function Show-UpdateForm {
    param (
        [Parameter(Mandatory)] $projects,
        [Parameter(Mandatory)] $packages
    )

    $form = New-Object Windows.Forms.Form
    $form.Text = "Update npm versions"
    $form.Size = New-Object Drawing.Size(600, 500)
    $form.StartPosition = "CenterScreen"

    $mainPanel = New-Object System.Windows.Forms.TableLayoutPanel
    $mainPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
    $mainPanel.RowCount = 2
    $mainPanel.ColumnCount = 1
    $mainPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Percent, 100)))
    $mainPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Absolute, 35)))
    $form.Controls.Add($mainPanel)

    

    $selectionPanel = New-Object System.Windows.Forms.TableLayoutPanel
    $selectionPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
    $selectionPanel.BorderStyle = 'Fixed3D'
    $selectionPanel.BackColor = 'White'
    $selectionPanel.RowCount = 2
    $selectionPanel.ColumnCount = 1
    $selectionPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Absolute, 85)))
    $selectionPanel.RowStyles.Add((New-Object System.Windows.Forms.RowStyle([System.Windows.Forms.SizeType]::Percent, 100)))
    $mainPanel.Controls.Add($selectionPanel, 0, 0)

    $selectProjectPanel = New-Object System.Windows.Forms.Panel
    $selectProjectPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
    $selectProjectPanel.Anchor = [System.Windows.Forms.AnchorStyles]::Top -bor [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Left -bor [System.Windows.Forms.AnchorStyles]::Right
    $selectionPanel.Controls.Add($selectProjectPanel, 0, 0)

    $y = 10

    $labelProgetto = New-Object Windows.Forms.Label
    $labelProgetto.Text = "Seleziona progetto:"
    $labelProgetto.Location = New-Object Drawing.Point(10, $y)
    $labelProgetto.Size = New-Object Drawing.Size(200, 20)
    $selectProjectPanel.Controls.Add($labelProgetto)
    $y += 20

    # ComboBox progetti
    $cmbProgetti = New-Object Windows.Forms.ComboBox
    $cmbProgetti.Location = New-Object Drawing.Point(10, $y)
    $cmbProgetti.Size = New-Object Drawing.Size(340, 30)
    $cmbProgetti.DropDownStyle = 'DropDownList'
    $cmbProgetti.Items.AddRange($projects)
    $selectProjectPanel.Controls.Add($cmbProgetti)
    $y += 35

    $labelPacchetti = New-Object Windows.Forms.Label
    $labelPacchetti.Text = "Seleziona i pacchetti da aggiornare:"
    $labelPacchetti.Location = New-Object Drawing.Point(10, $y)
    $labelPacchetti.Size = New-Object Drawing.Size(200, 20)
    $selectProjectPanel.Controls.Add($labelPacchetti)

    $y = 10
    # Panel per i checkbox pacchetti
    $pnlPackages = New-Object Windows.Forms.Panel
    $pnlPackages.AutoScroll = $true
    $pnlPackages.Anchor = [System.Windows.Forms.AnchorStyles]::Top -bor [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Left -bor [System.Windows.Forms.AnchorStyles]::Right
    $selectionPanel.Controls.Add($pnlPackages, 0, 1)

    function Update-PackageCheckboxes($projectName) {
        $pnlPackages.Controls.Clear()
        $script:chkboxes = @()
        if (-not $projectName) { return }
        $pkgJsonPath = Join-Path -Path $PSScriptRoot -ChildPath "..\..\..\..\pdca0\$projectName\package.json"
        $pkgJsonPath = [System.IO.Path]::GetFullPath($pkgJsonPath)
        if (-not (Test-Path $pkgJsonPath)) { return }
        try {
            $pkgContent = Get-Content $pkgJsonPath -Raw | ConvertFrom-Json
            $allDeps = @()
            if ($pkgContent.dependencies) { $allDeps += $pkgContent.dependencies.PSObject.Properties.Name }
            if ($pkgContent.devDependencies) { $allDeps += $pkgContent.devDependencies.PSObject.Properties.Name }
            $yChk = 0
            foreach ($dep in ($allDeps | Sort-Object -Unique)) {
                $chk = New-Object Windows.Forms.CheckBox
                $chk.Text = $dep
                $chk.Location = New-Object Drawing.Point(20, $yChk)
                $chk.Size = New-Object Drawing.Size(250, 20)
                $pnlPackages.Controls.Add($chk)
                $script:chkboxes += $chk
                $yChk += 22
            }
        } catch {}
    }

    # Aggiorna i checkbox quando cambia il progetto selezionato
    $cmbProgetti.Add_SelectedIndexChanged({
        $selected = $cmbProgetti.SelectedItem
        Update-PackageCheckboxes $selected
    })

    # Inizializza lista pacchetti vuota
    $script:chkboxes = @()
    Update-PackageCheckboxes $null

    $buttonPanel = New-Object System.Windows.Forms.Panel
    $buttonPanel.Dock = [System.Windows.Forms.DockStyle]::Fill
    $buttonPanel.Anchor = [System.Windows.Forms.AnchorStyles]::Top -bor [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Left -bor [System.Windows.Forms.AnchorStyles]::Right
    $buttonPanel.Height = 35
    $mainPanel.Controls.Add($buttonPanel, 0, 1)

    # Pulsante aggiorna
    $btnAggiorna = New-Object Windows.Forms.Button
    $btnAggiorna.Text = "Aggiorna"
    $btnAggiorna.Location = New-Object Drawing.Point(490, 0)
    $btnAggiorna.Size = New-Object Drawing.Size(80, 25)
    $btnAggiorna.Anchor = [System.Windows.Forms.AnchorStyles]::Bottom -bor [System.Windows.Forms.AnchorStyles]::Right
    $buttonPanel.Controls.Add($btnAggiorna)

    # Gestione click
    $btnAggiorna.Add_Click({
        $selectedProject = $cmbProgetti.SelectedItem
        $selectedPackages = $chkboxes | Where-Object { $_.Checked } | ForEach-Object { $_.Text }
        if (-not $selectedProject) {
            [System.Windows.Forms.MessageBox]::Show("Seleziona un progetto!", "Errore", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Error)
            return
        }
        if (-not $selectedPackages -or $selectedPackages.Count -eq 0) {
            [System.Windows.Forms.MessageBox]::Show("Seleziona almeno un pacchetto!", "Errore", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Error)
            return
        }

        # Lancia lo script update-npm-versions.ps1
        $scriptPath = Join-Path $ScriptDir 'update-npm-versions.ps1'
        $allArgs = @(
            '-NoProfile',
            '-ExecutionPolicy', 'Bypass',
            '-File', $scriptPath,
            '-ProjectFolder', $selectedProject,
            '-Packages'
        ) + ($selectedPackages -join ',')

        Start-Process powershell -ArgumentList $allArgs -Wait
        [System.Windows.Forms.MessageBox]::Show("Aggiornamento completato!", "Fatto", [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Information)
    })
    $buttonPanel.Controls.Add($btnAggiorna)

    $form.ShowDialog()
}

Show-UpdateForm -projects $projects -packages $packages
