param(
    [Parameter(Mandatory = $true, Position = 0)]
    [string]$ProjectFolder,
    [Parameter(Mandatory = $true, Position = 1)]
    [string[]]$Packages
)

$PackagesArray = $Packages -split ','

Write-Host "Aggiornamento pacchetti specifici all'ultima versione beta..." -ForegroundColor Cyan

if (-not $ProjectFolder) {
    Write-Error "Errore: Specificare il folder su cui eseguire l'aggiornamento dei pacchetti."
    Write-Host "Utilizzo: .\\update-lib.ps1 nome-folder-progetto nome-pacchetto1 nome-pacchetto2 ..."
    exit 1
}

if (-not $Packages -or $PackagesArray.Count -eq 0) {
    Write-Error "Errore: Specificare almeno un pacchetto da aggiornare."
    Write-Host "Utilizzo: .\\update-lib.ps1 nome-folder-progetto nome-pacchetto1 nome-pacchetto2 ..."
    exit 1
}

$module = Join-Path $PSScriptRoot "..\..\..\..\pdca0\$ProjectFolder"
Write-Host "Aggiorno $module" -ForegroundColor Yellow
Set-Location $module

if (-not (Test-Path "package.json")) {
    Write-Error "Errore: package.json non trovato nella directory corrente."
    exit 1
}

# Crea lo script Node.js temporaneo
$nodeScript = @'
const fs = require('fs');
const { execSync } = require('child_process');
const pkg = require('./package.json');
const packages = process.argv.slice(2);
async function updatePackages() {
  for (const packageName of packages) {
    try {
      console.log(`Aggiornamento di ${packageName} all'ultima versione beta...`);
      const versionsOutput = execSync(`npm view ${packageName} versions --json`, {encoding: 'utf8'});
      const versions = JSON.parse(versionsOutput);
      let latestVersion = versions[versions.length-1];
      const caret = !latestVersion.includes('snapshot') ? '' : '^';
      if (pkg.dependencies && pkg.dependencies[packageName]) {
        pkg.dependencies[packageName] = `${caret}${latestVersion}`;
      } else if (pkg.devDependencies && pkg.devDependencies[packageName]) {
        pkg.devDependencies[packageName] = `${caret}${latestVersion}`;
      }
      console.log(`Pacchetto ${packageName} aggiornato alla versione ${latestVersion}`);
    } catch (error) {
      console.error(`Errore nell'aggiornamento di ${packageName}: ${error.message}`);
    }
  }
  fs.writeFileSync('package.json', JSON.stringify(pkg, null, 2));
}
updatePackages();
'@

$jsFile = "update-npm-lib.js"
Set-Content -Path $jsFile -Value $nodeScript -Encoding UTF8

# Esegue lo script Node.js
node $jsFile @PackagesArray

# Rimuove il file temporaneo
Remove-Item $jsFile

Write-Host "Aggiornamento completato." -ForegroundColor Green
exit 0
