# Script di setup ambiente per smlink e htm_home
# NOTA: Alcune operazioni richiedono intervento manuale (vedi commenti)

# 1. Ottieni la home directory dell'utente
$home = [Environment]::GetFolderPath('UserProfile')

# 2. Crea la directory _symlink_home nella home
$symlinkHome = Join-Path $home "_symlink_home"
if (-not (Test-Path $symlinkHome)) {
    New-Item -ItemType Directory -Path $symlinkHome | Out-Null
    Write-Host "Creata directory: $symlinkHome"
} else {
    Write-Host "Directory già esistente: $symlinkHome"
}

# 3. Crea il file smlink-settings.json con placeholder
$settingsPath = Join-Path $symlinkHome "smlink-settings.json"
$jsonContent = @'{
    "credential-map": {
        "bfasscm01t.fideuram.bancafideuram.it": {
            "user": "VOSTRO_USERNAME",
            "password": "VOSTRA_PASSWORD"
        }
    }
}
'@
Set-Content -Path $settingsPath -Value $jsonContent -Encoding UTF8
Write-Host "Creato file: $settingsPath"

# 4. Scompatta smlink.zip nella _symlink_home
# Modifica il percorso di $smlinkZip se necessario
$smlinkZip = "C:\PDC_INSTALL\software\smlink.zip"
if (Test-Path $smlinkZip) {
    Expand-Archive -Path $smlinkZip -DestinationPath $symlinkHome -Force
    Write-Host "smlink.zip spacchettato in $symlinkHome"
} else {
    Write-Warning "File non trovato: $smlinkZip (modifica il percorso se necessario)"
}

# 5. Verifica permessi per la creazione di link simbolici
# Questo controllo verifica il gruppo Administrators, ma la policy può variare
$identity = [System.Security.Principal.WindowsIdentity]::GetCurrent()
$principal = New-Object System.Security.Principal.WindowsPrincipal($identity)
if ($principal.IsInRole([System.Security.Principal.WindowsBuiltInRole]::Administrator)) {
    Write-Host "L'utente attuale ha privilegi di amministratore (può creare symlink se la policy lo consente)."
} else {
    Write-Warning "L'utente attuale NON ha privilegi di amministratore. Potrebbero servire permessi aggiuntivi per creare symlink."
}
Write-Host "Verifica manualmente la policy: 'Criteri di sicurezza locali -> Criteri locali -> Assegnazione diritti utente -> Creazione di collegamenti simbolici'"

# 6. Crea la cartella _htm_home nella home dell'utente
$htmHome = Join-Path $home "_htm_home"
if (-not (Test-Path $htmHome)) {
    New-Item -ItemType Directory -Path $htmHome | Out-Null
    Write-Host "Creata directory: $htmHome"
} else {
    Write-Host "Directory già esistente: $htmHome"
}

# 7. Istruzioni per lanciare smlink manualmente
Write-Host "\n*** ATTENZIONE: Passaggi manuali necessari ***"
Write-Host "1. Apri CMD come amministratore (NON PowerShell), spostati nella cartella PDC (es: cd %userprofile%\\Projects\\puc)"
Write-Host "2. Lancia il comando: smlink -ck I/CT/ct-master -n"
Write-Host "   (verifica che il meno sia quello giusto, non un trattino lungo!)"

# 8. Istruzioni per copiare htm_home.zip
Write-Host "\nPer completare la configurazione, estrai la cartella 'hostname_NOMEMACCHINA' da htm_home.zip (in PDC_INSTALL/software) e copiala dentro $htmHome"
Write-Host "\nScript completato."
